/*-----------------------------------------------------------------------------------
**	$Id: HD_ceilfloor.c,v 1.2 2012/03/06 08:54:23 wolf Exp wolf $
**---------------------------------------------------------------------------------*/

#include "ceilfloor.h"

#ifdef DEBUG
#include <stdio.h>
#endif

#define	quad	long long

typedef union {
	double	d;
	quad	q;
} dq;

/*----------------------------------------------------------
**	Größte ganze Zahl kleiner oder gleich dem Argument
*---------------------------------------------------------*/

double
floor (double arg) {
	quad	mantisse;
	int	exp;
	int	nega;
	int	isInt;
	dq	extract;

	/*---------------------------------------
	**	Extract fields
	*--------------------------------------*/

	extract.d = arg;
	mantisse= extract.q & 0xfffffffffffffll;
	exp	= ((extract.q >> 52) & 0x7ff) - 1023;
	nega	= extract.q<0;

	/*---------------------------------------
	**	Debug
	*--------------------------------------*/

#ifdef DEBUG
	printf ("F %20.10f  %2i %013llx %4i\n", arg, nega, mantisse, exp);
#endif

	/*---------------------------------------
	**	Too big for fract or NaN
	*--------------------------------------*/

	if (exp>=52) return arg;

	/*---------------------------------------
	**	-1 < arg < 1
	*--------------------------------------*/

	if (exp<0) return nega ? -1.0 : 0.0;

	/*---------------------------------------
	**	|arg| >= 1 and may have fracts
	*--------------------------------------*/

	if ((mantisse & ((1ll << (52-exp)) - 1)) == 0)
		return arg;

	/*---------------------------------------
	**	|arg| >= 1 and has fracts
	*--------------------------------------*/

	extract.q &= (-1ll << (52-exp));

	return nega ? extract.d - 1.0 : extract.d;
}

/*----------------------------------------------------------
**	Kleinste ganze Zahl größer oder gleich dem Argument
*---------------------------------------------------------*/

double
ceil (double arg) {
	quad	mantisse;
	int	exp;
	int	nega;
	int	isInt;
	dq	extract;

	/*---------------------------------------
	**	Extract fields
	*--------------------------------------*/

	extract.d = arg;
	mantisse= extract.q & 0xfffffffffffffll;
	exp	= ((extract.q >> 52) & 0x7ff) - 1023;
	nega	= extract.q<0;

	/*---------------------------------------
	**	Debug
	*--------------------------------------*/

#ifdef DEBUG
	printf ("C %20.10f  %2i %013llx %4i\n", arg, nega, mantisse, exp);
#endif

	/*---------------------------------------
	**	Too big for fract or NaN
	*--------------------------------------*/

	if (exp>=52) return arg;

	/*---------------------------------------
	**	-1 < arg < 1
	*--------------------------------------*/

	if (exp<0) return nega ? 0.0 : 1.0;

	/*---------------------------------------
	**	|arg| >= 1 and may have fracts
	*--------------------------------------*/

	if ((mantisse & ((1ll << (52-exp)) - 1)) == 0)
		return arg;

	/*---------------------------------------
	**	|arg| >= 1 and has fracts
	*--------------------------------------*/

	extract.q &= (-1ll << (52-exp));

	return nega ? extract.d : extract.d + 1.0;
}

/*-----------------------------------------------------------------------------------
**	$Id: HD_ceilfloor.c,v 1.2 2012/03/06 08:54:23 wolf Exp wolf $
**---------------------------------------------------------------------------------*/
