/*-----------------------------------------------------------------------------------
**	$Id: sincos.c,v 1.3 2012/03/06 08:54:23 wolf Exp wolf $
**---------------------------------------------------------------------------------*/

#include "sincos.h"

extern
double floor (double arg);

extern
double sqrt (double arg);

/*---------------------------------------------------------------
**	Hilfsfunktionen mit eingeschränktem Wertebereich
**	-pi/4 .. +pi/4
**-------------------------------------------------------------*/

static double
_sin_ (double arg) {
	double	sum	= arg;
	double	msqarg	= -arg*arg;
	double	term	= arg;
	int	i;
	/*------------------------------------------------------
	** für höhere Genauigkeit umgekehrte Reihenfolge!
	**----------------------------------------------------*/
	for (i=2; i<=14; i+=2) {
		term *= msqarg / (i*(i+1));
		sum  += term;
	}
	return sum;
}

static double
_cos_ (double arg) {
	double s = _sin_ (arg);
	return sqrt (1.0 - s*s);
}

/*---------------------------------------------------------------
**	double hat 52 bit Genauigkeit,
**	daher Pi mit 16 Stellen
**-------------------------------------------------------------*/

#define	PIBY2	(3.141592653589793 / 2.0)

/*---------------------------------------------------------------
**	Funktionen mit Wertebereich ungefähr
**	-10^9 .. +10^9
**-------------------------------------------------------------*/

double
sin (double phi) {
	long	periode = (long) floor (phi / PIBY2 + 0.5);
	double	reduced = phi - PIBY2 * periode;
	switch (periode&3) {
	default:
		return _sin_ (reduced);
	case 1:
		return _cos_ (reduced);
	case 2:
		return -_sin_(reduced);
	case 3:
		return -_cos_(reduced);
	}
}

double
cos (double phi) {
	long	periode = floor (phi / PIBY2 + 0.125);
	double	reduced = phi - PIBY2 * periode;
	switch (periode&3) {
	default:
		return _cos_ (reduced);
	case 1:
		return -_sin_(reduced);
	case 2:
		return -_cos_(reduced);
	case 3:
		return _sin_ (reduced);
	}
}

/*-----------------------------------------------------------------------------------
**	$Id: sincos.c,v 1.3 2012/03/06 08:54:23 wolf Exp wolf $
**---------------------------------------------------------------------------------*/
